C Calculation of renormalization parameters for QED.model
C This file is part of aITALC
C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>

      subroutine renormalization_QED(active)
      implicit none
      logical active
      integer maxgenerations    !limit the sum over generations
                                !this number should be in concordance
                                !with the active fermions in QED.model
      parameter (maxgenerations=1)
      integer i1,i2
      double precision MLN(3),MLL(3),MQU(3),MQD(3)
      double precision MLN2(3),MLL2(3),MQU2(3),MQD2(3)
      double precision GVLN,GVLN2,GVLL,GVLL2,GVQU,GVQU2,GVQD,GVQD2
      double precision GALN,GALN2,GALL,GALL2,GAQU,GAQU2,GAQD,GAQD2
      double precision QLN,QLL,QQU,QQD
      double precision QLN2,QLL2,QQU2,QQD2
      double complex  SelfWT, SelfAAT, SelfAZT, SelfZAT, SelfZZT
      double complex DSelfWT,DSelfAAT,DSelfAZT,DSelfZAT,DSelfZZT
      double complex  SelfH, SelfGW, SelfG0
      double complex DSelfH,DSelfGW,DSelfG0
      double complex Tadpole
      
      double precision alpha,cw2,sw2

#include <variablelist.hf>
#include <looptools.h>
*#define DEBUG 1
C -----------------------------------------------------------------------
C At the very beginning we allow to set to zero all renormalization
C parameters and fields
      if (active) goto 10
      
      dQEDZAA=zero
      dQEDZe=zero
      
      do i1=1,maxgenerations
        do i2=1,4
          dQEDMf(i2,i1)=zero
          dQEDZfV(i2,i1)=zero
        enddo
      enddo

      return
      
C -----------------------------------------------------------------------
 10   continue

      MLN(1)=Mne
      MLN(2)=Mnm
      MLN(3)=Mnt
      MLL(1)=Mle
      MLL(2)=Mlm
      MLL(3)=Mlt
      MQU(1)=Mu
      MQU(2)=Mc
      MQU(3)=Mt
      MQD(1)=Md
      MQD(2)=Ms
      MQD(3)=Mb

      do i1=1,maxgenerations
        MLN2(i1)=MLN(i1)**2
        MLL2(i1)=MLL(i1)**2
        MQU2(i1)=MQU(i1)**2
        MQD2(i1)=MQD(i1)**2
      enddo

      QLN=Qne
      QLN2=QLN**2
      GALN=GAne
      GALN2=GAne**2
      GVLN=GVne
      GVLN2=GVne**2

      QLL=Qle
      QLL2=QLL**2
      GALL=GAle
      GALL2=GAle**2
      GVLL=GVle
      GVLL2=GVle**2

      QQU=Qu
      QQU2=QQU**2
      GAQU=GAu
      GAQU2=GAu**2
      GVQU=GVu
      GVQU2=GVu**2

      QQD=Qd
      QQD2=QQD**2
      GAQD=GAd
      GAQD2=GAd**2
      GVQD=GVd
      GVQD2=GVd**2


      do i1=1,maxgenerations

        dQEDMf(1,i1) = + MLN(i1)*(e/8/Pi)**2 * DBLE(
     &         + 0
     &         )

        dQEDMf(2,i1) = + MLL(i1)*(e/8/Pi)**2 * DBLE(
     &         + 4*QLL2
     &         - 8*QLL2*B1(MLL2(i1),MLL2(i1),MMA)
     &         - 16*QLL2*B0(MLL2(i1),MLL2(i1),MMA)
     &         )
      
        dQEDMf(3,i1) = + MQU(i1)*(e/8/Pi)**2 * DBLE(
     &         + 4*QQU2
     &         - 8*QQU2*B1(MQU2(i1),MQU2(i1),MMA)
     &         - 16*QQU2*B0(MQU2(i1),MQU2(i1),MMA)
     &         )
      
        dQEDMf(4,i1) = + MQD(i1)*(e/8/Pi)**2 * DBLE(
     &         + 4*QQD2
     &         - 8*QQD2*B1(MQD2(i1),MQD2(i1),MMA)
     &         - 16*QQD2*B0(MQD2(i1),MQD2(i1),MMA)
     &         )
      
      
C For type N neutrinos
        dQEDZfV(1,i1) = + (e/8/Pi)**2* DBLE (
     &         + 0
     &         )
      
C For Lepton type electron
        dQEDZfV(2,i1) = + (e/8/Pi)**2* DBLE (
     &         + MLL2(i1) * (
     &         + 16*DB1(MLL2(i1),MLL2(i1),MMA)*QLL2
     &         + 32*DB0(MLL2(i1),MLL2(i1),MMA)*QLL2
     &         )
     &         + 4*QLL2
     &         + 8*B1(MLL2(i1),MLL2(i1),MMA)*QLL2
     &         )
      
C For type Up Quarks
        dQEDZfV(3,i1) = + (e/8/Pi)**2* DBLE (
     &         + MQU2(i1) * (
     &         + 16*DB1(MQU2(i1),MQU2(i1),MMA)*QQU2
     &         + 32*DB0(MQU2(i1),MQU2(i1),MMA)*QQU2
     &         )
     &         + 4*QQU2
     &         + 8*B1(MQU2(i1),MQU2(i1),MMA)*QQU2
     &         )

C For Down type Quarks
        dQEDZfV(4,i1) = + (e/8/Pi)**2* DBLE (
     &         + MQD2(i1) * (
     &         + 16*DB1(MQD2(i1),MQD2(i1),MMA)*QQD2
     &         + 32*DB0(MQD2(i1),MQD2(i1),MMA)*QQD2
     &         )
     &         + 4*QQD2
     &         + 8*B1(MQD2(i1),MQD2(i1),MMA)*QQD2
     &         )

      enddo

C End of fermion mass and field renormalization
C -----------------------------------------------------------------------
      SelfAAT=0
      DSelfAAT=0

C In the current QED.model there are no quarks running.

      do i1 = 1,maxgenerations
        SelfAAT = SelfAAT + (e**2*Pi**(-2)/48) * (
     &       - 8*A0(MLL2(i1))*QLL2
c     &       - 24*A0(MQU2(i1))*QQU2
c     &       - 24*A0(MQD2(i1))*QQD2
     &       + 8*B0(MMA,MLL2(i1),MLL2(i1))*MLL2(i1)*QLL2
c     &       + 24*B0(MMA,MQU2(i1),MQU2(i1))*MQU2(i1)*QQU2
c     &       + 24*B0(MMA,MQD2(i1),MQD2(i1))*MQD2(i1)*QQD2
     &       + 8*MLL2(i1)*QLL2
c     &       + 24*MQU2(i1)*QQU2
c     &       + 24*MQD2(i1)*QQD2
     &       )
        DSelfAAT = DSelfAAT + (e**2*Pi**(-2)/48) * (
     &       + 4*B0(MMA,MLL2(i1),MLL2(i1))*QLL2
c     &       + 12*B0(MMA,MQU2(i1),MQU2(i1))*QQU2
c     &       + 12*B0(MMA,MQD2(i1),MQD2(i1))*QQD2
     &       + 8*DB0(MMA,MLL2(i1),MLL2(i1))*MLL2(i1)*QLL2
c     &       + 24*DB0(MMA,MQU2(i1),MQU2(i1))*MQU2(i1)*QQU2
c     &       + 24*DB0(MMA,MQD2(i1),MQD2(i1))*MQD2(i1)*QQD2
     &       )
      enddo

C -----------------------------------------------------------------------
        dQEDZAA=-DBLE(DSelfAAT)

C Eq (3.32)
        dQEDZe = 0.5D0*DBLE(-dQEDZAA)





#ifdef DEBUG
      do i1=1,4
        do i2=1,maxgenerations
          print 11,' dQEDMf(',i1,',',i2,')=', dQEDMf(i1,i2)
          print 11,'dQEDZfV(',i1,',',i2,')=', dQEDZfV(i1,i2)
        enddo
      enddo
 11   format (A5,I1,A1,I1,A2,1X,E22.16)

      print *, 'Tadpole =', Tadpole
      print *, ' SelfAAT=', SelfAAT
      print *, 'DSelfAAT=',DSelfAAT

      print *, 'dQEDZAA =', dQEDZAA

      print *, 'dtad =', dtad
      print *, 'dQEDZe  =', dQEDZe
    
#endif

       return
       end
